# Copyright (c) 2017-2026 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.unknown_field_set import UnknownFieldSet


T = TypeVar("T", bound="Duration")


@_attrs_define
class Duration:
    """
    Attributes:
        seconds (int):
        nanos (int):
        unknown_fields (UnknownFieldSet | Unset):
    """

    seconds: int
    nanos: int
    unknown_fields: UnknownFieldSet | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        seconds = self.seconds

        nanos = self.nanos

        unknown_fields: dict[str, Any] | Unset = UNSET
        if not isinstance(self.unknown_fields, Unset):
            unknown_fields = self.unknown_fields.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "seconds": seconds,
                "nanos": nanos,
            }
        )
        if unknown_fields is not UNSET:
            field_dict["unknownFields"] = unknown_fields

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.unknown_field_set import UnknownFieldSet

        d = dict(src_dict)
        seconds = d.pop("seconds")

        nanos = d.pop("nanos")

        _unknown_fields = d.pop("unknownFields", UNSET)
        unknown_fields: UnknownFieldSet | Unset
        if isinstance(_unknown_fields, Unset):
            unknown_fields = UNSET
        else:
            unknown_fields = UnknownFieldSet.from_dict(_unknown_fields)

        duration = cls(
            seconds=seconds,
            nanos=nanos,
            unknown_fields=unknown_fields,
        )

        duration.additional_properties = d
        return duration

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright (c) 2017-2026 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file

from .standalone_bft_ordering_service_pb2 import Ordered, ReadOrderedRequest, ReadOrderedResponse, SendRequest, SendResponse
from .standalone_bft_ordering_service_pb2_grpc import StandaloneBftOrderingServiceStub

__all__ = [
    "Ordered",
    "ReadOrderedRequest",
    "ReadOrderedResponse",
    "SendRequest",
    "SendResponse",
    "StandaloneBftOrderingServiceStub",
]



# Copyright (c) 2017-2026 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.participant_authorization_topology_format import (
        ParticipantAuthorizationTopologyFormat,
    )


T = TypeVar("T", bound="TopologyFormat")


@_attrs_define
class TopologyFormat:
    """A format specifying which topology transactions to include and how to render them.

    Attributes:
        include_participant_authorization_events (ParticipantAuthorizationTopologyFormat | Unset): A format specifying
            which participant authorization topology transactions to include and how to render them.
    """

    include_participant_authorization_events: (
        ParticipantAuthorizationTopologyFormat | Unset
    ) = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        include_participant_authorization_events: dict[str, Any] | Unset = UNSET
        if not isinstance(self.include_participant_authorization_events, Unset):
            include_participant_authorization_events = (
                self.include_participant_authorization_events.to_dict()
            )

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if include_participant_authorization_events is not UNSET:
            field_dict["includeParticipantAuthorizationEvents"] = (
                include_participant_authorization_events
            )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.participant_authorization_topology_format import (
            ParticipantAuthorizationTopologyFormat,
        )

        d = dict(src_dict)
        _include_participant_authorization_events = d.pop(
            "includeParticipantAuthorizationEvents", UNSET
        )
        include_participant_authorization_events: (
            ParticipantAuthorizationTopologyFormat | Unset
        )
        if isinstance(_include_participant_authorization_events, Unset):
            include_participant_authorization_events = UNSET
        else:
            include_participant_authorization_events = (
                ParticipantAuthorizationTopologyFormat.from_dict(
                    _include_participant_authorization_events
                )
            )

        topology_format = cls(
            include_participant_authorization_events=include_participant_authorization_events,
        )

        topology_format.additional_properties = d
        return topology_format

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright (c) 2017-2026 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: com/digitalasset/canton/sequencer/admin/v30/sequencer_bft_additional_snapshot_info.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'com/digitalasset/canton/sequencer/admin/v30/sequencer_bft_additional_snapshot_info.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nXcom/digitalasset/canton/sequencer/admin/v30/sequencer_bft_additional_snapshot_info.proto\x12+com.digitalasset.canton.sequencer.admin.v30\"\x9f\t\n\"BftSequencerSnapshotAdditionalInfo\x12\x99\x01\n\x14sequencers_active_at\x18\x01 \x03(\x0b\x32g.com.digitalasset.canton.sequencer.admin.v30.BftSequencerSnapshotAdditionalInfo.SequencersActiveAtEntryR\x12sequencersActiveAt\x1a\xa8\x01\n\x17SequencersActiveAtEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12w\n\x05value\x18\x02 \x01(\x0b\x32\x61.com.digitalasset.canton.sequencer.admin.v30.BftSequencerSnapshotAdditionalInfo.SequencerActiveAtR\x05value:\x02\x38\x01\x1a\xb1\x06\n\x11SequencerActiveAt\x12\x1c\n\ttimestamp\x18\x01 \x01(\x03R\ttimestamp\x12\x31\n\x12start_epoch_number\x18\x02 \x01(\x03H\x00R\x10startEpochNumber\x88\x01\x01\x12L\n!first_block_number_in_start_epoch\x18\x03 \x01(\x03H\x01R\x1c\x66irstBlockNumberInStartEpoch\x88\x01\x01\x12S\n$start_epoch_topology_query_timestamp\x18\x04 \x01(\x03H\x02R startEpochTopologyQueryTimestamp\x88\x01\x01\x12\\\n)start_epoch_could_alter_ordering_topology\x18\x05 \x01(\x08H\x03R$startEpochCouldAlterOrderingTopology\x88\x01\x01\x12/\n\x11previous_bft_time\x18\x06 \x01(\x03H\x04R\x0fpreviousBftTime\x88\x01\x01\x12Y\n\'previous_epoch_topology_query_timestamp\x18\x07 \x01(\x03H\x05R#previousEpochTopologyQueryTimestamp\x88\x01\x01\x12\x46\n\x1dleader_selection_policy_state\x18\x08 \x01(\x0cH\x06R\x1aleaderSelectionPolicyState\x88\x01\x01\x42\x15\n\x13_start_epoch_numberB$\n\"_first_block_number_in_start_epochB\'\n%_start_epoch_topology_query_timestampB,\n*_start_epoch_could_alter_ordering_topologyB\x14\n\x12_previous_bft_timeB*\n(_previous_epoch_topology_query_timestampB \n\x1e_leader_selection_policy_state\"\x9f\x07\n#BlacklistLeaderSelectionPolicyState\x12!\n\x0c\x65poch_number\x18\x01 \x01(\x03R\x0b\x65pochNumber\x12,\n\x12start_block_number\x18\x02 \x01(\x03R\x10startBlockNumber\x12}\n\tblacklist\x18\x03 \x03(\x0b\x32_.com.digitalasset.canton.sequencer.admin.v30.BlacklistLeaderSelectionPolicyState.BlacklistEntryR\tblacklist\x1a\x9e\x01\n\x0e\x42lacklistEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12v\n\x05value\x18\x02 \x01(\x0b\x32`.com.digitalasset.canton.sequencer.admin.v30.BlacklistLeaderSelectionPolicyState.BlacklistStatusR\x05value:\x02\x38\x01\x1a\x86\x04\n\x0f\x42lacklistStatus\x12\x85\x01\n\x08on_trial\x18\x01 \x01(\x0b\x32h.com.digitalasset.canton.sequencer.admin.v30.BlacklistLeaderSelectionPolicyState.BlacklistStatus.OnTrialH\x00R\x07onTrial\x12\x90\x01\n\x0b\x62lacklisted\x18\x02 \x01(\x0b\x32l.com.digitalasset.canton.sequencer.admin.v30.BlacklistLeaderSelectionPolicyState.BlacklistStatus.BlacklistedH\x00R\x0b\x62lacklisted\x1aJ\n\x07OnTrial\x12?\n\x1c\x66\x61iled_attempts_before_trial\x18\x01 \x01(\x03R\x19\x66\x61iledAttemptsBeforeTrial\x1a\x81\x01\n\x0b\x42lacklisted\x12\x34\n\x16\x66\x61iled_attempts_before\x18\x01 \x01(\x03R\x14\x66\x61iledAttemptsBefore\x12<\n\x1b\x65pochs_left_until_new_trial\x18\x02 \x01(\x03R\x17\x65pochsLeftUntilNewTrialB\x08\n\x06statusB\\ZZgithub.com/digital-asset/dazl-client/v8/go/api/com/digitalasset/canton/sequencer/admin/v30b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'com.digitalasset.canton.sequencer.admin.v30.sequencer_bft_additional_snapshot_info_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZZgithub.com/digital-asset/dazl-client/v8/go/api/com/digitalasset/canton/sequencer/admin/v30'
  _globals['_BFTSEQUENCERSNAPSHOTADDITIONALINFO_SEQUENCERSACTIVEATENTRY']._loaded_options = None
  _globals['_BFTSEQUENCERSNAPSHOTADDITIONALINFO_SEQUENCERSACTIVEATENTRY']._serialized_options = b'8\001'
  _globals['_BLACKLISTLEADERSELECTIONPOLICYSTATE_BLACKLISTENTRY']._loaded_options = None
  _globals['_BLACKLISTLEADERSELECTIONPOLICYSTATE_BLACKLISTENTRY']._serialized_options = b'8\001'
  _globals['_BFTSEQUENCERSNAPSHOTADDITIONALINFO']._serialized_start=138
  _globals['_BFTSEQUENCERSNAPSHOTADDITIONALINFO']._serialized_end=1321
  _globals['_BFTSEQUENCERSNAPSHOTADDITIONALINFO_SEQUENCERSACTIVEATENTRY']._serialized_start=333
  _globals['_BFTSEQUENCERSNAPSHOTADDITIONALINFO_SEQUENCERSACTIVEATENTRY']._serialized_end=501
  _globals['_BFTSEQUENCERSNAPSHOTADDITIONALINFO_SEQUENCERACTIVEAT']._serialized_start=504
  _globals['_BFTSEQUENCERSNAPSHOTADDITIONALINFO_SEQUENCERACTIVEAT']._serialized_end=1321
  _globals['_BLACKLISTLEADERSELECTIONPOLICYSTATE']._serialized_start=1324
  _globals['_BLACKLISTLEADERSELECTIONPOLICYSTATE']._serialized_end=2251
  _globals['_BLACKLISTLEADERSELECTIONPOLICYSTATE_BLACKLISTENTRY']._serialized_start=1572
  _globals['_BLACKLISTLEADERSELECTIONPOLICYSTATE_BLACKLISTENTRY']._serialized_end=1730
  _globals['_BLACKLISTLEADERSELECTIONPOLICYSTATE_BLACKLISTSTATUS']._serialized_start=1733
  _globals['_BLACKLISTLEADERSELECTIONPOLICYSTATE_BLACKLISTSTATUS']._serialized_end=2251
  _globals['_BLACKLISTLEADERSELECTIONPOLICYSTATE_BLACKLISTSTATUS_ONTRIAL']._serialized_start=2035
  _globals['_BLACKLISTLEADERSELECTIONPOLICYSTATE_BLACKLISTSTATUS_ONTRIAL']._serialized_end=2109
  _globals['_BLACKLISTLEADERSELECTIONPOLICYSTATE_BLACKLISTSTATUS_BLACKLISTED']._serialized_start=2112
  _globals['_BLACKLISTLEADERSELECTIONPOLICYSTATE_BLACKLISTSTATUS_BLACKLISTED']._serialized_end=2241
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2026 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.created_event import CreatedEvent


T = TypeVar("T", bound="JsAssignedEvent")


@_attrs_define
class JsAssignedEvent:
    """Records that a contract has been assigned, and it can be used on the target synchronizer.

    Attributes:
        source (str): The ID of the source synchronizer.
            Must be a valid synchronizer id.
            Required
        target (str): The ID of the target synchronizer.
            Must be a valid synchronizer id.
            Required
        reassignment_id (str): The ID from the unassigned event.
            For correlation capabilities.
            Must be a valid LedgerString (as described in ``value.proto``).
            Required
        submitter (str): Party on whose behalf the assign command was executed.
            Empty if the assignment happened offline via the repair service.
            Must be a valid PartyIdString (as described in ``value.proto``).
            Optional
        reassignment_counter (int): Each corresponding assigned and unassigned event has the same reassignment_counter.
            This strictly increases
            with each unassign command for the same contract. Creation of the contract corresponds to reassignment_counter
            equals zero.
            Required
        created_event (CreatedEvent): Records that a contract has been created, and choices may now be exercised on it.
    """

    source: str
    target: str
    reassignment_id: str
    submitter: str
    reassignment_counter: int
    created_event: CreatedEvent
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        source = self.source

        target = self.target

        reassignment_id = self.reassignment_id

        submitter = self.submitter

        reassignment_counter = self.reassignment_counter

        created_event = self.created_event.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "source": source,
                "target": target,
                "reassignmentId": reassignment_id,
                "submitter": submitter,
                "reassignmentCounter": reassignment_counter,
                "createdEvent": created_event,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.created_event import CreatedEvent

        d = dict(src_dict)
        source = d.pop("source")

        target = d.pop("target")

        reassignment_id = d.pop("reassignmentId")

        submitter = d.pop("submitter")

        reassignment_counter = d.pop("reassignmentCounter")

        created_event = CreatedEvent.from_dict(d.pop("createdEvent"))

        js_assigned_event = cls(
            source=source,
            target=target,
            reassignment_id=reassignment_id,
            submitter=submitter,
            reassignment_counter=reassignment_counter,
            created_event=created_event,
        )

        js_assigned_event.additional_properties = d
        return js_assigned_event

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright (c) 2017-2026 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from . import topology_manager_read_service_pb2 as com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2

GRPC_GENERATED_VERSION = '1.76.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + ' but the generated code in com/digitalasset/canton/topology/admin/v30/topology_manager_read_service_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class TopologyManagerReadServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListNamespaceDelegation = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListNamespaceDelegation',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListNamespaceDelegationRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListNamespaceDelegationResponse.FromString,
                _registered_method=True)
        self.ListDecentralizedNamespaceDefinition = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListDecentralizedNamespaceDefinition',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListDecentralizedNamespaceDefinitionRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListDecentralizedNamespaceDefinitionResponse.FromString,
                _registered_method=True)
        self.ListOwnerToKeyMapping = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListOwnerToKeyMapping',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListOwnerToKeyMappingRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListOwnerToKeyMappingResponse.FromString,
                _registered_method=True)
        self.ListPartyToKeyMapping = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListPartyToKeyMapping',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyToKeyMappingRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyToKeyMappingResponse.FromString,
                _registered_method=True)
        self.ListSynchronizerTrustCertificate = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListSynchronizerTrustCertificate',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerTrustCertificateRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerTrustCertificateResponse.FromString,
                _registered_method=True)
        self.ListParticipantSynchronizerPermission = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListParticipantSynchronizerPermission',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListParticipantSynchronizerPermissionRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListParticipantSynchronizerPermissionResponse.FromString,
                _registered_method=True)
        self.ListPartyHostingLimits = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListPartyHostingLimits',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyHostingLimitsRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyHostingLimitsResponse.FromString,
                _registered_method=True)
        self.ListVettedPackages = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListVettedPackages',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListVettedPackagesRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListVettedPackagesResponse.FromString,
                _registered_method=True)
        self.ListPartyToParticipant = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListPartyToParticipant',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyToParticipantRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyToParticipantResponse.FromString,
                _registered_method=True)
        self.ListSynchronizerParametersState = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListSynchronizerParametersState',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerParametersStateRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerParametersStateResponse.FromString,
                _registered_method=True)
        self.ListMediatorSynchronizerState = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListMediatorSynchronizerState',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListMediatorSynchronizerStateRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListMediatorSynchronizerStateResponse.FromString,
                _registered_method=True)
        self.ListSequencerSynchronizerState = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListSequencerSynchronizerState',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSequencerSynchronizerStateRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSequencerSynchronizerStateResponse.FromString,
                _registered_method=True)
        self.ListSynchronizerUpgradeAnnouncement = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListSynchronizerUpgradeAnnouncement',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerUpgradeAnnouncementRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerUpgradeAnnouncementResponse.FromString,
                _registered_method=True)
        self.ListSequencerConnectionSuccessor = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListSequencerConnectionSuccessor',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSequencerConnectionSuccessorRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSequencerConnectionSuccessorResponse.FromString,
                _registered_method=True)
        self.ListAvailableStores = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListAvailableStores',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListAvailableStoresRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListAvailableStoresResponse.FromString,
                _registered_method=True)
        self.ListAll = channel.unary_unary(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListAll',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListAllRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListAllResponse.FromString,
                _registered_method=True)
        self.ExportTopologySnapshot = channel.unary_stream(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ExportTopologySnapshot',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ExportTopologySnapshotRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ExportTopologySnapshotResponse.FromString,
                _registered_method=True)
        self.ExportTopologySnapshotV2 = channel.unary_stream(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ExportTopologySnapshotV2',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ExportTopologySnapshotV2Request.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ExportTopologySnapshotV2Response.FromString,
                _registered_method=True)
        self.GenesisState = channel.unary_stream(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/GenesisState',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.GenesisStateRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.GenesisStateResponse.FromString,
                _registered_method=True)
        self.GenesisStateV2 = channel.unary_stream(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/GenesisStateV2',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.GenesisStateV2Request.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.GenesisStateV2Response.FromString,
                _registered_method=True)
        self.LogicalUpgradeState = channel.unary_stream(
                '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/LogicalUpgradeState',
                request_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.LogicalUpgradeStateRequest.SerializeToString,
                response_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.LogicalUpgradeStateResponse.FromString,
                _registered_method=True)


class TopologyManagerReadServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ListNamespaceDelegation(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListDecentralizedNamespaceDefinition(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListOwnerToKeyMapping(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListPartyToKeyMapping(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListSynchronizerTrustCertificate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListParticipantSynchronizerPermission(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListPartyHostingLimits(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListVettedPackages(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListPartyToParticipant(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListSynchronizerParametersState(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListMediatorSynchronizerState(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListSequencerSynchronizerState(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListSynchronizerUpgradeAnnouncement(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListSequencerConnectionSuccessor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListAvailableStores(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListAll(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExportTopologySnapshot(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExportTopologySnapshotV2(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GenesisState(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GenesisStateV2(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LogicalUpgradeState(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TopologyManagerReadServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ListNamespaceDelegation': grpc.unary_unary_rpc_method_handler(
                    servicer.ListNamespaceDelegation,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListNamespaceDelegationRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListNamespaceDelegationResponse.SerializeToString,
            ),
            'ListDecentralizedNamespaceDefinition': grpc.unary_unary_rpc_method_handler(
                    servicer.ListDecentralizedNamespaceDefinition,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListDecentralizedNamespaceDefinitionRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListDecentralizedNamespaceDefinitionResponse.SerializeToString,
            ),
            'ListOwnerToKeyMapping': grpc.unary_unary_rpc_method_handler(
                    servicer.ListOwnerToKeyMapping,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListOwnerToKeyMappingRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListOwnerToKeyMappingResponse.SerializeToString,
            ),
            'ListPartyToKeyMapping': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPartyToKeyMapping,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyToKeyMappingRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyToKeyMappingResponse.SerializeToString,
            ),
            'ListSynchronizerTrustCertificate': grpc.unary_unary_rpc_method_handler(
                    servicer.ListSynchronizerTrustCertificate,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerTrustCertificateRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerTrustCertificateResponse.SerializeToString,
            ),
            'ListParticipantSynchronizerPermission': grpc.unary_unary_rpc_method_handler(
                    servicer.ListParticipantSynchronizerPermission,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListParticipantSynchronizerPermissionRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListParticipantSynchronizerPermissionResponse.SerializeToString,
            ),
            'ListPartyHostingLimits': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPartyHostingLimits,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyHostingLimitsRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyHostingLimitsResponse.SerializeToString,
            ),
            'ListVettedPackages': grpc.unary_unary_rpc_method_handler(
                    servicer.ListVettedPackages,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListVettedPackagesRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListVettedPackagesResponse.SerializeToString,
            ),
            'ListPartyToParticipant': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPartyToParticipant,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyToParticipantRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyToParticipantResponse.SerializeToString,
            ),
            'ListSynchronizerParametersState': grpc.unary_unary_rpc_method_handler(
                    servicer.ListSynchronizerParametersState,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerParametersStateRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerParametersStateResponse.SerializeToString,
            ),
            'ListMediatorSynchronizerState': grpc.unary_unary_rpc_method_handler(
                    servicer.ListMediatorSynchronizerState,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListMediatorSynchronizerStateRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListMediatorSynchronizerStateResponse.SerializeToString,
            ),
            'ListSequencerSynchronizerState': grpc.unary_unary_rpc_method_handler(
                    servicer.ListSequencerSynchronizerState,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSequencerSynchronizerStateRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSequencerSynchronizerStateResponse.SerializeToString,
            ),
            'ListSynchronizerUpgradeAnnouncement': grpc.unary_unary_rpc_method_handler(
                    servicer.ListSynchronizerUpgradeAnnouncement,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerUpgradeAnnouncementRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerUpgradeAnnouncementResponse.SerializeToString,
            ),
            'ListSequencerConnectionSuccessor': grpc.unary_unary_rpc_method_handler(
                    servicer.ListSequencerConnectionSuccessor,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSequencerConnectionSuccessorRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSequencerConnectionSuccessorResponse.SerializeToString,
            ),
            'ListAvailableStores': grpc.unary_unary_rpc_method_handler(
                    servicer.ListAvailableStores,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListAvailableStoresRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListAvailableStoresResponse.SerializeToString,
            ),
            'ListAll': grpc.unary_unary_rpc_method_handler(
                    servicer.ListAll,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListAllRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListAllResponse.SerializeToString,
            ),
            'ExportTopologySnapshot': grpc.unary_stream_rpc_method_handler(
                    servicer.ExportTopologySnapshot,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ExportTopologySnapshotRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ExportTopologySnapshotResponse.SerializeToString,
            ),
            'ExportTopologySnapshotV2': grpc.unary_stream_rpc_method_handler(
                    servicer.ExportTopologySnapshotV2,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ExportTopologySnapshotV2Request.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ExportTopologySnapshotV2Response.SerializeToString,
            ),
            'GenesisState': grpc.unary_stream_rpc_method_handler(
                    servicer.GenesisState,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.GenesisStateRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.GenesisStateResponse.SerializeToString,
            ),
            'GenesisStateV2': grpc.unary_stream_rpc_method_handler(
                    servicer.GenesisStateV2,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.GenesisStateV2Request.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.GenesisStateV2Response.SerializeToString,
            ),
            'LogicalUpgradeState': grpc.unary_stream_rpc_method_handler(
                    servicer.LogicalUpgradeState,
                    request_deserializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.LogicalUpgradeStateRequest.FromString,
                    response_serializer=com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.LogicalUpgradeStateResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class TopologyManagerReadService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ListNamespaceDelegation(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListNamespaceDelegation',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListNamespaceDelegationRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListNamespaceDelegationResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListDecentralizedNamespaceDefinition(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListDecentralizedNamespaceDefinition',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListDecentralizedNamespaceDefinitionRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListDecentralizedNamespaceDefinitionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListOwnerToKeyMapping(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListOwnerToKeyMapping',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListOwnerToKeyMappingRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListOwnerToKeyMappingResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListPartyToKeyMapping(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListPartyToKeyMapping',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyToKeyMappingRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyToKeyMappingResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListSynchronizerTrustCertificate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListSynchronizerTrustCertificate',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerTrustCertificateRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerTrustCertificateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListParticipantSynchronizerPermission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListParticipantSynchronizerPermission',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListParticipantSynchronizerPermissionRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListParticipantSynchronizerPermissionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListPartyHostingLimits(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListPartyHostingLimits',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyHostingLimitsRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyHostingLimitsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListVettedPackages(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListVettedPackages',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListVettedPackagesRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListVettedPackagesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListPartyToParticipant(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListPartyToParticipant',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyToParticipantRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListPartyToParticipantResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListSynchronizerParametersState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListSynchronizerParametersState',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerParametersStateRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerParametersStateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListMediatorSynchronizerState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListMediatorSynchronizerState',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListMediatorSynchronizerStateRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListMediatorSynchronizerStateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListSequencerSynchronizerState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListSequencerSynchronizerState',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSequencerSynchronizerStateRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSequencerSynchronizerStateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListSynchronizerUpgradeAnnouncement(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListSynchronizerUpgradeAnnouncement',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerUpgradeAnnouncementRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSynchronizerUpgradeAnnouncementResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListSequencerConnectionSuccessor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListSequencerConnectionSuccessor',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSequencerConnectionSuccessorRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListSequencerConnectionSuccessorResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListAvailableStores(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListAvailableStores',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListAvailableStoresRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListAvailableStoresResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListAll(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ListAll',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListAllRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ListAllResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ExportTopologySnapshot(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ExportTopologySnapshot',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ExportTopologySnapshotRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ExportTopologySnapshotResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ExportTopologySnapshotV2(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/ExportTopologySnapshotV2',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ExportTopologySnapshotV2Request.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.ExportTopologySnapshotV2Response.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GenesisState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/GenesisState',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.GenesisStateRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.GenesisStateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GenesisStateV2(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/GenesisStateV2',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.GenesisStateV2Request.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.GenesisStateV2Response.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def LogicalUpgradeState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/com.digitalasset.canton.topology.admin.v30.TopologyManagerReadService/LogicalUpgradeState',
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.LogicalUpgradeStateRequest.SerializeToString,
            com_dot_digitalasset_dot_canton_dot_topology_dot_admin_dot_v30_dot_topology__manager__read__service__pb2.LogicalUpgradeStateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



# Copyright (c) 2017-2026 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.experimental_features import ExperimentalFeatures
    from ..models.offset_checkpoint_feature import OffsetCheckpointFeature
    from ..models.package_feature import PackageFeature
    from ..models.party_management_feature import PartyManagementFeature
    from ..models.user_management_feature import UserManagementFeature


T = TypeVar("T", bound="FeaturesDescriptor")


@_attrs_define
class FeaturesDescriptor:
    """
    Attributes:
        experimental (ExperimentalFeatures | Unset): See the feature message definitions for descriptions.
        user_management (UserManagementFeature | Unset):
        party_management (PartyManagementFeature | Unset):
        offset_checkpoint (OffsetCheckpointFeature | Unset):
        package_feature (PackageFeature | Unset):
    """

    experimental: ExperimentalFeatures | Unset = UNSET
    user_management: UserManagementFeature | Unset = UNSET
    party_management: PartyManagementFeature | Unset = UNSET
    offset_checkpoint: OffsetCheckpointFeature | Unset = UNSET
    package_feature: PackageFeature | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        experimental: dict[str, Any] | Unset = UNSET
        if not isinstance(self.experimental, Unset):
            experimental = self.experimental.to_dict()

        user_management: dict[str, Any] | Unset = UNSET
        if not isinstance(self.user_management, Unset):
            user_management = self.user_management.to_dict()

        party_management: dict[str, Any] | Unset = UNSET
        if not isinstance(self.party_management, Unset):
            party_management = self.party_management.to_dict()

        offset_checkpoint: dict[str, Any] | Unset = UNSET
        if not isinstance(self.offset_checkpoint, Unset):
            offset_checkpoint = self.offset_checkpoint.to_dict()

        package_feature: dict[str, Any] | Unset = UNSET
        if not isinstance(self.package_feature, Unset):
            package_feature = self.package_feature.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if experimental is not UNSET:
            field_dict["experimental"] = experimental
        if user_management is not UNSET:
            field_dict["userManagement"] = user_management
        if party_management is not UNSET:
            field_dict["partyManagement"] = party_management
        if offset_checkpoint is not UNSET:
            field_dict["offsetCheckpoint"] = offset_checkpoint
        if package_feature is not UNSET:
            field_dict["packageFeature"] = package_feature

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.experimental_features import ExperimentalFeatures
        from ..models.offset_checkpoint_feature import OffsetCheckpointFeature
        from ..models.package_feature import PackageFeature
        from ..models.party_management_feature import PartyManagementFeature
        from ..models.user_management_feature import UserManagementFeature

        d = dict(src_dict)
        _experimental = d.pop("experimental", UNSET)
        experimental: ExperimentalFeatures | Unset
        if isinstance(_experimental, Unset):
            experimental = UNSET
        else:
            experimental = ExperimentalFeatures.from_dict(_experimental)

        _user_management = d.pop("userManagement", UNSET)
        user_management: UserManagementFeature | Unset
        if isinstance(_user_management, Unset):
            user_management = UNSET
        else:
            user_management = UserManagementFeature.from_dict(_user_management)

        _party_management = d.pop("partyManagement", UNSET)
        party_management: PartyManagementFeature | Unset
        if isinstance(_party_management, Unset):
            party_management = UNSET
        else:
            party_management = PartyManagementFeature.from_dict(_party_management)

        _offset_checkpoint = d.pop("offsetCheckpoint", UNSET)
        offset_checkpoint: OffsetCheckpointFeature | Unset
        if isinstance(_offset_checkpoint, Unset):
            offset_checkpoint = UNSET
        else:
            offset_checkpoint = OffsetCheckpointFeature.from_dict(_offset_checkpoint)

        _package_feature = d.pop("packageFeature", UNSET)
        package_feature: PackageFeature | Unset
        if isinstance(_package_feature, Unset):
            package_feature = UNSET
        else:
            package_feature = PackageFeature.from_dict(_package_feature)

        features_descriptor = cls(
            experimental=experimental,
            user_management=user_management,
            party_management=party_management,
            offset_checkpoint=offset_checkpoint,
            package_feature=package_feature,
        )

        features_descriptor.additional_properties = d
        return features_descriptor

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright (c) 2017-2026 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.signature import Signature


T = TypeVar("T", bound="SignedTransaction")


@_attrs_define
class SignedTransaction:
    """
    Attributes:
        transaction (str):
        signatures (list[Signature] | Unset):
    """

    transaction: str
    signatures: list[Signature] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        transaction = self.transaction

        signatures: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.signatures, Unset):
            signatures = []
            for signatures_item_data in self.signatures:
                signatures_item = signatures_item_data.to_dict()
                signatures.append(signatures_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "transaction": transaction,
            }
        )
        if signatures is not UNSET:
            field_dict["signatures"] = signatures

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.signature import Signature

        d = dict(src_dict)
        transaction = d.pop("transaction")

        _signatures = d.pop("signatures", UNSET)
        signatures: list[Signature] | Unset = UNSET
        if _signatures is not UNSET:
            signatures = []
            for signatures_item_data in _signatures:
                signatures_item = Signature.from_dict(signatures_item_data)

                signatures.append(signatures_item)

        signed_transaction = cls(
            transaction=transaction,
            signatures=signatures,
        )

        signed_transaction.additional_properties = d
        return signed_transaction

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright (c) 2017-2026 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off
# isort: skip_file
from ...health.v30 import status_service_pb2 as _status_service_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class MediatorStatusRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MediatorStatusResponse(_message.Message):
    __slots__ = ("status", "not_initialized")
    class MediatorStatusResponseStatus(_message.Message):
        __slots__ = ("common_status", "physical_synchronizer_id", "protocol_version")
        COMMON_STATUS_FIELD_NUMBER: _ClassVar[int]
        PHYSICAL_SYNCHRONIZER_ID_FIELD_NUMBER: _ClassVar[int]
        PROTOCOL_VERSION_FIELD_NUMBER: _ClassVar[int]
        common_status: _status_service_pb2.Status
        physical_synchronizer_id: str
        protocol_version: int
        def __init__(self, common_status: _Optional[_Union[_status_service_pb2.Status, _Mapping]] = ..., physical_synchronizer_id: _Optional[str] = ..., protocol_version: _Optional[int] = ...) -> None: ...
    STATUS_FIELD_NUMBER: _ClassVar[int]
    NOT_INITIALIZED_FIELD_NUMBER: _ClassVar[int]
    status: MediatorStatusResponse.MediatorStatusResponseStatus
    not_initialized: _status_service_pb2.NotInitialized
    def __init__(self, status: _Optional[_Union[MediatorStatusResponse.MediatorStatusResponseStatus, _Mapping]] = ..., not_initialized: _Optional[_Union[_status_service_pb2.NotInitialized, _Mapping]] = ...) -> None: ...



# Copyright (c) 2017-2026 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from asyncio import new_event_loop, run_coroutine_threadsafe, set_event_loop, sleep
from concurrent.futures import Future
from queue import Queue
import sys
from threading import Thread
from types import TracebackType
from typing import Callable, Optional

from ..aio import Connection, QueryStream

if sys.version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self

__all__ = ["ConnectionThunk", "QueryStreamThunk"]


class ConnectionThunk:
    """
    A blocking Connection wrapper around an asynchronous connection.
    """

    def __init__(self, conn_fn: Callable[[], Connection], *, name: Optional[str] = None) -> None:
        self._conn_fn = conn_fn
        self._conn = None
        self._thread = Thread(target=self._main, name=name, daemon=True)
        self._loop = new_event_loop()
        self._fut = self._loop.create_future()

    def _main(self) -> None:
        """
        The "main" thread that runs the event loop where operations for this connection are scheduled.
        """
        set_event_loop(self._loop)
        self._loop.run_until_complete(self._fut)

    def open(self) -> None:
        # start the main thread
        self._thread.start()

        fut = run_coroutine_threadsafe(self._create_conn(), self._loop)
        fut.result()

    async def _create_conn(self):
        await sleep(0)
        self._conn = self._conn_fn()
        await self._conn.open()

    def close(self) -> None:
        conn = self._conn
        if conn is not None:
            run_coroutine_threadsafe(conn.close(), self._loop).result()
        self._loop.call_soon_threadsafe(self._fut.set_result, None)
        self._thread.join()

    def __enter__(self) -> Self:
        self.open()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def create(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.create(*args, **kwargs), self._loop)
        return fut.result()

    def create_and_exercise(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.create_and_exercise(*args, **kwargs), self._loop)
        return fut.result()

    def exercise(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.exercise(*args, **kwargs), self._loop)
        return fut.result()

    def exercise_by_key(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.exercise_by_key(*args, **kwargs), self._loop)
        return fut.result()

    def submit(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.submit(*args, **kwargs), self._loop)
        return fut.result()

    def archive(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.archive(*args, **kwargs), self._loop)
        return fut.result()

    def archive_by_key(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.archive_by_key(*args, **kwargs), self._loop)
        return fut.result()

    def get_ledger_end(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.get_ledger_end(*args, **kwargs), self._loop)
        return fut.result()

    def query(self, *args, **kwargs):
        return QueryStreamThunk(self._conn.query(*args, **kwargs), self._loop)

    def query_many(self, *args, **kwargs):
        return QueryStreamThunk(self._conn.query_many(*args, **kwargs), self._loop)

    def stream(self, *args, **kwargs):
        return QueryStreamThunk(self._conn.stream(*args, **kwargs), self._loop)

    def stream_many(self, *args, **kwargs):
        return QueryStreamThunk(self._conn.stream_many(*args, **kwargs), self._loop)

    def get_user(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.get_user(*args, **kwargs), self._loop)
        return fut.result()

    def list_users(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.list_users(*args, **kwargs), self._loop)
        return fut.result()

    def allocate_party(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.allocate_party(*args, **kwargs), self._loop)
        return fut.result()

    def list_known_parties(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.list_known_parties(*args, **kwargs), self._loop)
        return fut.result()

    def upload_package(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.upload_package(*args, **kwargs), self._loop)
        return fut.result()

    def get_package(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.get_package(*args, **kwargs), self._loop)
        return fut.result()

    def list_package_ids(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.list_package_ids(*args, **kwargs), self._loop)
        return fut.result()

    def get_metering_report(self, *args, **kwargs):
        fut = run_coroutine_threadsafe(self._conn.get_metering_report(*args, **kwargs), self._loop)
        return fut.result()


class QueryStreamThunk:
    """
    A blocking QueryStream wrapper around an asynchronous query stream.
    """

    def __init__(self, stream: QueryStream, loop):
        self._loop = loop
        self._stream = stream
        self._q = Queue()  # type: ignore
        self._fut: Future[None] | None = None

    def __enter__(self):
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType,
    ) -> None:
        # try to abort an existing iteration if one is running
        if self._fut is not None:
            try:
                self._fut.cancel()
            except Exception:  # noqa
                pass
        self.close()

    def on_create(self, *args):
        return self._stream.on_create(*args)

    def on_archive(self, *args):
        return self._stream.on_archive(*args)

    def on_boundary(self, *args):
        return self._stream.on_boundary(*args)

    def close(self) -> None:
        self._q.put_nowait(None)

    def run(self) -> None:
        self._fut = run_coroutine_threadsafe(self._stream.run(), self._loop)
        self._fut.result()

    def creates(self):
        self._fut = run_coroutine_threadsafe(self._producer(self._stream.creates), self._loop)
        return self._consume_queue()

    def events(self):
        self._fut = run_coroutine_threadsafe(self._producer(self._stream.events), self._loop)
        return self._consume_queue()

    def items(self):
        self._fut = run_coroutine_threadsafe(self._producer(self._stream.items), self._loop)
        return self._consume_queue()

    def __iter__(self):
        return self.items()

    async def _producer(self, source):
        async with self._stream:
            async for item in source():
                self._q.put_nowait(item)
            self._q.put_nowait(None)

    def _consume_queue(self):
        while True:
            item = self._q.get()
            if item is not None:
                yield item
            else:
                break
